"""
Home facts:
    Home type
    Beds, Full baths, 3/4 baths, 1/2 baths, 1/4 baths
    Finished squared feet
    Lot size
    Year Built
    Structural remodel year
    Openhouse
    Additional Info


"""

